"""pynomer - A python wrapper for nomer"""

from .pynomer import *

__version__ = "0.1.2"
__author__ = "nleguillarme <nicolas.leguillarme@univ-grenoble-alpes.fr>"
__all__ = [
    "version",
    "clean",
    "input_schema",
    "output_schema",
    "properties",
    "matchers",
    "validate_term",
    "validate_term_link",
    "replace",
    "append",
]
